-------------------------------------------------------------------------------
-- SmartSell
-- Created by Aeldra (EU-Proudmoore)
--
-- Adds the vendor selling value and additional information to the item tooltips.
-------------------------------------------------------------------------------

local Title = "SmartSell";
local Version = "v3.0a";

local SmartSell = {};
local L = SMARTSELL_LOCALS;
local isInit = false;
local TitleVersion = Title.." "..Version;


-- Returns a chat color code string
local function BCC(r, g, b)
  return string.format("|cff%02x%02x%02x", (r*255), (g*255), (b*255));
end

local BL  = BCC(0.1, 0.1, 1.0);
local BLD = BCC(0.0, 0.0, 0.7);
local BLL = BCC(0.5, 0.8, 1.0);
local GR  = BCC(0.1, 1.0, 0.1);
local GRD = BCC(0.0, 0.7, 0.0);
local GRL = BCC(0.25, 0.75, 0.25);
local RD  = BCC(1.0, 0.1, 0.1);
local RDD = BCC(0.7, 0.0, 0.0);
local RDL = BCC(1.0, 0.3, 0.3);
local YL  = BCC(1.0, 1.0, 0.0);
local YLD = BCC(0.7, 0.7, 0.0);
local YLL = BCC(1.0, 1.0, 0.5);
local OR  = BCC(1.0, 0.5, 0.25);
local ORD = BCC(0.7, 0.5, 0.0);
local ORL = BCC(1.0, 0.6, 0.3);
local WH  = BCC(1.0, 1.0, 1.0);
local CY  = BCC(0.5, 1.0, 1.0);
local GY  = BCC(0.5, 0.5, 0.5);
local GYD = BCC(0.35, 0.35, 0.35);
local GYL = BCC(0.65, 0.65, 0.65);
local GBL = BCC(0.2, 1.0, 0.6);
local GOL = BCC(1.0, 0.84, 0.0);
local SIL = BCC(0.78, 0.78, 0.78);
local COP = BCC(0.84, 0.5, 0.37);


local function bShow(b)
  if (b == nil or not b) then
    return RD.."Off";
  else
    return GR.."On";    
  end
end


local function Msg(txt, showAddon)
  if (showAddon) then
    if (txt == "") then
      print(GBL..TitleVersion);
    else
	    print(GBL..Title..YLL..format(": %s", tostring(txt)));
	  end
  else
	  print(YLL..tostring(txt));
  end
end


local sm_g;
local function SplitMoney(money)
	sm_g = floor(money / (COPPER_PER_SILVER * SILVER_PER_GOLD));
	return sm_g, floor((money - (sm_g * COPPER_PER_SILVER * SILVER_PER_GOLD)) / COPPER_PER_SILVER), money % COPPER_PER_SILVER;
end


local gmt_g, gmt_s, gmt_c, gmt_t;
local function GetMoneyText(money, style)
	gmt_t = "";
	
	if (style == 3 or style == 4) then	  
	  gmt_g, gmt_s, gmt_c = SplitMoney(money);
	  --print(gmt_g.."/"..gmt_s.."/"..gmt_c);
	  if (gmt_g > 0) then
		  gmt_t = WH..gmt_g..GOL.."g";
	  end

	  if (gmt_s > 0) then
		  if (gmt_t ~= "") then gmt_t = gmt_t.." " end
		  gmt_t = gmt_t..WH..gmt_s..SIL.."s";
	  end

	  if (gmt_c > 0) then
	    if (gmt_t ~= "") then gmt_t = gmt_t.." " end
		  gmt_t = gmt_t..WH..gmt_c..COP.."c";
	  end
	  
	elseif (style == 5 or style == 6) then
	  if (money >= 10000) then
		  gmt_t = WH..format("%.2f", money/10000)..GOL.."g";
	  elseif (money >= 100) then
		  gmt_t = WH..format("%.2f", money/100)..SIL.."s";
	  else
		  gmt_t = WH..format("%d", money)..COP.."c";
	  end
	end
	
	return gmt_t;
end


local itemName;
local itemLink;
local itemLevel;
local itemStackCount;
local itemID;
local itemValue;
local displayCount;
local sty;
local s, t;

local function AddToTooltip(self, itemLinkId, itemCount, bHideOnVendor)
  if (not isInit) then
    SmartSell:Init();
  end
  
  if (itemLinkId == nil) then
    return;
  end
  
	itemName, itemLink, _, itemLevel, _, _, _, itemStackCount = GetItemInfo(itemLinkId);
	if (itemLevel == nil) then
		return;
	end  
  
  displayCount = itemCount;
  if (itemCount == nil or itemCount == 0) then
    itemCount = 1;
    displayCount = 0;
  end
  
  if (not SmartSellDB.ShowStackValue) then
    itemCount = 1;
  end
  
	if (itemName ~= nil) then
	  s = "";
	  t = "";
	  sty = SmartSellDB.Style;
	  
		if (SmartSellDB.ShowItemLevel) then
		  if (sty == 1) then
		    self:AddLine(L.ItemLevel..WH..itemLevel.."|r");
		  else
		    s = "lvl"..WH..itemLevel.."|r";
			end
		end
  	
		if (SmartSellDB.ShowItemStackCount and itemStackCount > 1 and itemStackCount < 10000) then
		  if (sty == 1) then
		    self:AddLine(L.ItemStackCount..WH..displayCount.."|r/"..WH..itemStackCount.."|r");
		  else
		    if (s ~= "") then s = s.." - " end
		    s = s.."stk"..WH..displayCount.."|r/"..WH..itemStackCount.."|r";			  
			end
		end
  	
		_, _, itemID = string.find(itemLink, "item:(%d+)");
		if (itemID ~= nil) then
			if (SmartSellDB.ShowItemID) then
			  if (sty == 1) then
			    self:AddLine(L.ItemID..WH..itemID.."|r");
		    else
		      if (s ~= "") then s = s.." - " end
			    s = s.."id"..WH..itemID.."|r";			    
			  end
			end
			
			if (sty == 2) then
			  self:AddLine(s);
			end
  		
			itemID = tonumber(itemID);
			if (GetSellValue ~= nil and itemID ~= nil) then
			  itemValue = GetSellValue(itemID);
				if (itemValue == nil) then
				  if (sty == 1 or sty == 2) then
				    self:AddLine(L.ValueUnknown);				    
		      else
					  t = RD.."?";
					end
				else				  
				  if (itemValue == 0) then
				    if (sty == 1 or sty == 2) then
				      self:AddLine(L.Unsalable);
		        else
				      t = OR.."x";
				    end
				  else
				    if (MerchantFrame:IsVisible() and bHideOnVendor) then
				      -- do nothing
				    else
				      if (SmartSellDB.ShowItemValue) then
				        if (sty == 1 or sty == 2) then
				          SetTooltipMoney(self, itemValue * itemCount);				          
		            else		              
		              t = GetMoneyText(itemValue * itemCount, sty);
					      end
					    end
					  end					    
					end
				end
			end
		end				
		
		if (sty > 2) then			
		  if (s == "") then s = " " end
		  if (t == "") then t = " " end
		  if (sty == 4 or sty == 6) then
		    self:AddDoubleLine(s, t);		    
		  else
		    self:AddDoubleLine(t, s);
		  end
		end		
		self:Show();
	end

end


local iLink;
local iCount;

local function SetBagItem(self, bagID, slot)
  if (SmartSellDB.QuestOnly) then return end
	iLink = GetContainerItemLink(bagID, slot);
	if (iLink ~= nil) then
		_, iCount = GetContainerItemInfo(bagID, slot);
		AddToTooltip(self, iLink, iCount, true);
	end
end

local function SetInventoryItem(self, unit, slot, nameOnly)
  if (SmartSellDB.QuestOnly) then return end
	iLink = GetInventoryItemLink(unit, slot);
	if (iLink ~= nil) then
		iCount = GetInventoryItemCount(unit, slot);
		AddToTooltip(self, iLink, iCount);
	end
end

local function SetMerchantItem(self, index)
  if (SmartSellDB.QuestOnly) then return end
	iLink = GetMerchantItemLink(index);
	if (iLink ~= nil) then
		_, _, _, iCount = GetMerchantItemInfo(index);
		AddToTooltip(self, iLink, iCount, true);
	end
end

local function SetQuestItem(self, type, index)
	iLink = GetQuestItemLink(type, index);
	if (iLink ~= nil) then
	  _, _, iCount = GetQuestItemInfo(type, index);
	  AddToTooltip(self, iLink, iCount);
	end
end

local function SetQuestLogItem(self, type, index)
	iLink = GetQuestLogItemLink(type, index);
	if (iLink ~= nil) then
	  if (type == "choice") then
		  _, _, iCount = GetQuestLogChoiceInfo(index);
	  else
		  _, _, iCount = GetQuestLogRewardInfo(index);
	  end
	  AddToTooltip(self, iLink, iCount);
	end
end

local function SetTradeSkillItem(self, skill, id)
  if (SmartSellDB.QuestOnly) then return end
	if (id) then
		iLink = GetTradeSkillReagentItemLink(skill, id);
		_, _, iCount = GetTradeSkillReagentInfo(skill, id);
	else
		iLink = GetTradeSkillItemLink(skill);
		iCount = GetTradeSkillNumMade(skill);
	end
	if (iLink ~= nil) then
	  AddToTooltip(self, iLink, iCount)
	end
end

local function SetGuildBankItem(self, tab, slot)
  if (SmartSellDB.QuestOnly) then return end
	iLink = GetGuildBankItemLink(tab, slot);
	if (iLink ~= nil) then
		_, iCount = GetGuildBankItemInfo(tab, slot);
		AddToTooltip(self, iLink, iCount);
	end
end

local function SetLootItem(self, slot)
  if (SmartSellDB.QuestOnly) then return end
	if (LootSlotIsItem(slot)) then
		iLink = GetLootSlotLink(slot);
		if (iLink ~= nil) then
			_, _, iCount = GetLootSlotInfo(slot);
			AddToTooltip(self, iLink, iCount);
		end
	end
end

local function SetAuctionItem(self, type, index)
  if (SmartSellDB.QuestOnly) then return end
	iLink = GetAuctionItemLink(type, index);
	if (iLink ~= nil) then
		_, _, iCount = GetAuctionItemInfo(type, index);
		AddToTooltip(self, iLink, iCount);
	end
end

local function SetAuctionSellItem()
  if (SmartSellDB.QuestOnly) then return end
	_, iLink = GetItemInfo(GetAuctionSellItemInfo());
	if (iLink ~= nil) then
		_, _, iCount = GetAuctionSellItemInfo();
		AddToTooltip(self, iLink, iCount);
	end
end

local function SetInboxItem(self, index, attachIndex)
  if (SmartSellDB.QuestOnly) then return end
	iLink = GetInboxItemLink(index, attachIndex);
	if (iLink ~= nil) then
		_, _, iCount = GetInboxItem(index, attachIndex);
		AddToTooltip(self, iLink, iCount);
	end
end

local function SetSendMailItem(self, index)
  if (SmartSellDB.QuestOnly) then return end
	iLink = GetSendMailItemLink(index);
	if (iLink ~= nil) then
		_, _, iCount = GetSendMailItem(index);
		AddToTooltip(self, iLink, iCount);
	end
end

local iType;
local function SetAction(self, slot)
  if (SmartSellDB.QuestOnly) then return end
	iType, iLink = GetActionInfo(slot);
	if (iType ~= nil and iType == "item" and iLink ~= nil) then
  	iCount = GetActionCount(slot);
  	if (iCount < 1) then iCount = 1 end
		AddToTooltip(self, iLink, iCount);
	end
end


local function SetHyperlink(self, item)
  if (SmartSellDB.QuestOnly) then return end
	_, iLink = GetItemInfo(item);
	if (iLink ~= nil) then
		AddToTooltip(self, iLink, 1);
	end
end


local invSlot;
local function SetHyperlinkCompareItem(self, link, index)
  if (SmartSellDB.QuestOnly) then return end
	_, _, _, _, _, _, _, _, iEquipLoc = GetItemInfo(link);	
	if (iEquipLoc == nil or iEquipLoc == "" or iEquipLoc == "INVTYPE_BAG" or iEquipLoc == "INVTYPE_QUIVER") then
		return;
	end
	
	invSlot = nil;
	if (iEquipLoc == "INVTYPE_HEAD") then
		invSlot = GetInventorySlotInfo("HeadSlot");
	elseif (iEquipLoc == "INVTYPE_NECK") then
		invSlot = GetInventorySlotInfo("NeckSlot");
	elseif (iEquipLoc == "INVTYPE_SHOULDER") then
		invSlot = GetInventorySlotInfo("ShoulderSlot");
	elseif (iEquipLoc == "INVTYPE_CLOAK") then
		invSlot = GetInventorySlotInfo("BackSlot");
	elseif (iEquipLoc == "INVTYPE_CHEST" or iEquipLoc == "INVTYPE_ROBE") then
		invSlot = GetInventorySlotInfo("ChestSlot");
	elseif (iEquipLoc == "INVTYPE_BODY") then
		invSlot = GetInventorySlotInfo("ShirtSlot");
	elseif (iEquipLoc == "INVTYPE_TABARD") then
		invSlot = GetInventorySlotInfo("TabardSlot");
	elseif (iEquipLoc == "INVTYPE_WRIST") then
		invSlot = GetInventorySlotInfo("WristSlot");
	elseif (iEquipLoc == "INVTYPE_HAND") then
		invSlot = GetInventorySlotInfo("HandsSlot");
	elseif (iEquipLoc == "INVTYPE_WAIST") then
		invSlot = GetInventorySlotInfo("WaistSlot");
	elseif (iEquipLoc == "INVTYPE_LEGS") then
		invSlot = GetInventorySlotInfo("LegsSlot");
	elseif (iEquipLoc == "INVTYPE_FEET") then
		invSlot = GetInventorySlotInfo("FeetSlot");		
	elseif (iEquipLoc == "INVTYPE_FINGER") then
		_, iLink = self:GetItem();
		invSlot = GetInventorySlotInfo("Finger0Slot");
		if (iLink ~= GetInventoryItemLink("player", invSlot)) then
			invSlot = GetInventorySlotInfo("Finger1Slot");
		end
	elseif (iEquipLoc == "INVTYPE_TRINKET") then
		_, iLink = self:GetItem();
		invSlot = GetInventorySlotInfo("Trinket0Slot");
		if (iLink ~= GetInventoryItemLink("player", invSlot)) then
			invSlot = GetInventorySlotInfo("Trinket1Slot");
		end
	elseif (iEquipLoc == "INVTYPE_2HWEAPON" or iEquipLoc == "INVTYPE_HOLDABLE" or iEquipLoc == "INVTYPE_SHIELD"
	  or iEquipLoc == "INVTYPE_WEAPON" or iEquipLoc == "INVTYPE_WEAPONMAINHAND" or iEquipLoc == "INVTYPE_WEAPONOFFHAND") then
		_, iLink = self:GetItem();
		invSlot = GetInventorySlotInfo("MainHandSlot");
		if (iLink ~= GetInventoryItemLink("player", invSlot)) then
			invSlot = GetInventorySlotInfo("SecondaryHandSlot");
		end
	elseif (iEquipLoc == "INVTYPE_RANGED" or iEquipLoc == "INVTYPE_RANGEDRIGHT" or iEquipLoc == "INVTYPE_RELIC" or iEquipLoc == "INVTYPE_THROWN") then
		invSlot = GetInventorySlotInfo("RangedSlot");
	elseif (iEquipLoc == "INVTYPE_AMMO") then
		invSlot = GetInventorySlotInfo("AmmoSlot");
	end
	
	if (invSlot ~= nil) then
		iLink = GetInventoryItemLink("player", invSlot);
		if (iLink ~= nil) then
			iCount = GetInventoryItemCount("player", invSlot);
			if (self == ShoppingTooltip1 or self == ShoppingTooltip2 or self == ShoppingTooltip3) then
			  GameTooltip_ClearMoney(self);
			end
			AddToTooltip(self, iLink, iCount);
		end
  end
end


function SmartSell:Init()
  if (SmartSellDB == nil) then
    SmartSell:LoadDefaults();
  end

  if (SmartSellDB.Style == nil) then
    SmartSellDB.Style = 1;
  end
  if (SmartSellDB.ShowItemLevel == nil) then
    SmartSellDB.ShowItemLevel = true;
  end
  if (SmartSellDB.ShowItemID == nil) then
    SmartSellDB.ShowItemID = false;
  end
  if (SmartSellDB.ShowItemStackCount == nil) then
    SmartSellDB.ShowItemStackCount = true;
  end
  if (SmartSellDB.ShowItemValue == nil) then
    SmartSellDB.ShowItemValue = true;
  end
  if (SmartSellDB.QuestOnly == nil) then
    SmartSellDB.QuestOnly = false;
  end
  if (SmartSellDB.ShowStackValue == nil) then
    SmartSellDB.ShowStackValue = true;
  end
  
  isInit = true;
end


function SmartSell:LoadDefaults()
	SmartSellDB = {};
	SmartSellDB.Style = 1;
	SmartSellDB.ShowItemLevel = true;
	SmartSellDB.ShowItemID = false;
	SmartSellDB.ShowItemStackCount = true;
	SmartSellDB.ShowItemValue = true;
	SmartSellDB.QuestOnly = false;
	SmartSellDB.ShowStackValue = true;
end


function SmartSell_Command(txt)
  if (not isInit) then
    SmartSell:Init();
  end

  if(txt == "help" or txt == "?" or txt == "") then
    Msg("", true);
    Msg("Syntax: /ssell [command] or /smartsell [command]");
    Msg(GBL.."1-6"..YLL.."   -  ".."Change display style, current style = "..WH..tostring(SmartSellDB.Style));
    Msg(GBL.."sty"..YLL.."    -  ".."Cycles through all styles, current style = "..WH..tostring(SmartSellDB.Style));
    Msg(GBL.."lvl"..YLL.."     -  ".."Toggle display of the item level = "..bShow(SmartSellDB.ShowItemLevel));
    Msg(GBL.."id"..YLL.."      -  ".."Toggle display of the item id = "..bShow(SmartSellDB.ShowItemID));
    Msg(GBL.."sc"..YLL.."      -  ".."Toggle display of the item stack count = "..bShow(SmartSellDB.ShowItemStackCount));
    Msg(GBL.."sp"..YLL.."      -  ".."Toggle display of the stack price = "..bShow(SmartSellDB.ShowStackValue));
    Msg(GBL.."val"..YLL.."    -  ".."Toggle display of the item value = "..bShow(SmartSellDB.ShowItemValue));    
    Msg(GBL.."qo"..YLL.."     -  ".."Toggle display of info only for quest rewards = "..bShow(SmartSellDB.QuestOnly));
    Msg(GBL.."rs"..YLL.."      -  ".."Reset all settings to default");    
  elseif (txt == "lvl") then
    SmartSellDB.ShowItemLevel = not SmartSellDB.ShowItemLevel;
    Msg("Show item level = "..bShow(SmartSellDB.ShowItemLevel));
  elseif (txt == "id") then
    SmartSellDB.ShowItemID = not SmartSellDB.ShowItemID;
    Msg("Show item id = "..bShow(SmartSellDB.ShowItemID));
  elseif (txt == "sc") then
    SmartSellDB.ShowItemStackCount = not SmartSellDB.ShowItemStackCount;
    Msg("Show item stack count = "..bShow(SmartSellDB.ShowItemStackCount));
  elseif (txt == "sp") then
    SmartSellDB.ShowStackValue = not SmartSellDB.ShowStackValue;
    Msg("Show stack price = "..bShow(SmartSellDB.ShowStackValue));    
  elseif (txt == "val") then
    SmartSellDB.ShowItemValue = not SmartSellDB.ShowItemValue;
    Msg("Show item value = "..bShow(SmartSellDB.ShowItemValue));
  elseif (txt == "qo") then
    SmartSellDB.QuestOnly = not SmartSellDB.QuestOnly;
    Msg("Show info only for quest rewards = "..bShow(SmartSellDB.QuestOnly));    
  elseif (txt == "rs") then
    SmartSell:LoadDefaults();
    Msg("Reset SmartSell to default");
  elseif (txt == "sty") then
    SmartSellDB.Style = SmartSellDB.Style + 1;
    if (SmartSellDB.Style > 6) then SmartSellDB.Style = 1 end
    Msg("Style = "..GR..tostring(SmartSellDB.Style));
  elseif (tonumber(txt) ~= nil) then
    local i = tonumber(txt);
    if (i < 1 or i > 6) then i = 1 end
    SmartSellDB.Style = i;
    Msg("Style = "..GR..tostring(i));
  end
end


hooksecurefunc(GameTooltip, "SetBagItem", SetBagItem);
hooksecurefunc(GameTooltip, "SetInventoryItem", SetInventoryItem);
hooksecurefunc(GameTooltip, "SetMerchantItem", SetMerchantItem);
hooksecurefunc(GameTooltip, "SetQuestItem", SetQuestItem);
hooksecurefunc(GameTooltip, "SetQuestLogItem", SetQuestLogItem);
hooksecurefunc(GameTooltip, "SetTradeSkillItem", SetTradeSkillItem);
hooksecurefunc(GameTooltip, "SetGuildBankItem", SetGuildBankItem);
hooksecurefunc(GameTooltip, "SetLootItem", SetLootItem);
hooksecurefunc(GameTooltip, "SetAuctionItem", SetAuctionItem);
hooksecurefunc(GameTooltip, "SetAuctionSellItem", SetAuctionSellItem);
hooksecurefunc(GameTooltip, "SetInboxItem", SetInboxItem);
hooksecurefunc(GameTooltip, "SetSendMailItem", SetSendMailItem);
--hooksecurefunc(GameTooltip, "SetAction", SetAction);
hooksecurefunc(ItemRefTooltip, "SetHyperlink", SetHyperlink);
hooksecurefunc(ShoppingTooltip1, "SetHyperlinkCompareItem", SetHyperlinkCompareItem);
hooksecurefunc(ShoppingTooltip2, "SetHyperlinkCompareItem", SetHyperlinkCompareItem);
hooksecurefunc(ShoppingTooltip3, "SetHyperlinkCompareItem", SetHyperlinkCompareItem);

SlashCmdList["SMARTSELL"] = SmartSell_Command;
SLASH_SMARTSELL1 = "/ssell";
SLASH_SMARTSELL2 = "/smartsell";

Msg(TitleVersion.." loaded");